#lang racket

(require math/bigfloat math/flonum)
(require "syntax/types.rkt")

(provide (struct-out representation) get-representation *output-repr* *var-reprs* representation-type)
(module+ internals (provide define-representation))

;; Global precision tacking
(define *output-repr* (make-parameter '()))
(define *var-reprs* (make-parameter '()))

;; Structs

(struct representation
  (name type
   bf->repr repr->bf ordinal->repr repr->ordinal
   total-bits special-values)
  #:methods gen:custom-write
  [(define (write-proc repr port mode)
     (fprintf port "#<representation ~a>" (representation-name repr)))])

(define representations (make-hash))
(define (get-representation name)
  (hash-ref representations name
            (λ () (error 'get-representation "Unknown representation ~a" name))))

(define-syntax-rule (define-representation (name type) args ...)
  (begin
    (define name (representation 'name (get-type 'type) args ...))
    (hash-set! representations 'name name)))

(define-representation (bool bool)
  identity
  identity
  (λ (x) (= x 0))
  (λ (x) (if x 1 0))
  1
  null)

(define (shift bits fn)
  (define shift-val (expt 2 bits))
  (λ (x) (fn (- x shift-val))))

(define (unshift bits fn)
  (define shift-val (expt 2 bits))
  (λ (x) (+ (fn x) shift-val)))

(define-representation (binary64 real)
  bigfloat->flonum
  bf
  (shift 63 ordinal->flonum)
  (unshift 63 flonum->ordinal)
  64
  '(+nan.0 +inf.0 -inf.0))

(define (single-flonum->bit-field x)
  (integer-bytes->integer (real->floating-point-bytes x 4) #f))

(define (single-flonum->ordinal x)
  (cond
    [(< x 0.0f0) (- (single-flonum->bit-field (- 0.0f0 x)))]
    [else (single-flonum->bit-field (abs x))]))

(define (bit-field->single-flonum x)
  (real->single-flonum (floating-point-bytes->real (integer->integer-bytes x 4 #f) #f)))

(define (ordinal->single-flonum x)
  (cond
    [(< x 0) (- (bit-field->single-flonum (- x)))]
    [else (bit-field->single-flonum x)]))

(define (single-flonum-step x n)
  (ordinal->single-flonum (+ (single-flonum->ordinal x) n)))

(define (bigfloat->single-flonum x)
  (define loprec (parameterize ([bf-precision 24]) (bf+ 0.bf x)))
  (define y (real->single-flonum (bigfloat->flonum loprec)))
  (define x2 (bf y))
  (match (bf-rounding-mode)
    ['nearest y]
    ['up   (if (bf< x2 x) (single-flonum-step y 1) y)]
    ['down (if (bf> x2 x) (single-flonum-step y -1) y)]
    ['zero (if (bf< x 0.bf)
               (if (bf< x2 x) (single-flonum-step y 1) y)
               (if (bf> x2 x) (single-flonum-step y -1) y))]))

(define-representation (binary32 real)
  bigfloat->single-flonum
  bf
  ordinal->single-flonum
  single-flonum->ordinal
  32
  '(+nan.f +inf.f -inf.f))

